// Unified token -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "Token.h"
#include "VariableSet.h"

/** @file Token.C
 * Tokens used in the unification process
 */

Token::~Token ()
{
  delete myUnifiedUndefined;
}

bool
Token::isBindable (const class Valuation& valuation)
{
  assert (valuation.isOK ());
  if (class Value* value = myUnifier.m->getToken ()->eval (valuation)) {
    assert (valuation.isOK ());
    myIterator = myPlaceMarking.find (value);
    delete value;
    if (myIterator != end ()) {
      if (myCardinality <= PlaceMarking::getCount (myIterator))
	return true;
      myIterator = begin ();
    }
  }

  return false;
}

bool
Token::isBindableUnfold (const class Valuation& valuation)
{
  assert (valuation.isOK ());
  if (class Value* value = myUnifier.m->getToken ()->eval (valuation)) {
    assert (valuation.isOK ());
    myIterator = myPlaceMarking.find (value);
    delete value;
    return myIterator != end ();
  }

  return false;
}

void
Token::undefine (class Valuation& valuation)
{
  if (!myUnifier.vars)
    return;
  const class VariableSet& vars = *myUnifier.vars;
  VariableSet::const_iterator i = vars.begin ();
  if (!myUnifier.varMult)
    for (; i != vars.end (); i++)
      valuation.undefine (**i);
  else
    for (; i != vars.end (); i++)
      if (!isUnified (**i)) valuation.undefine (**i);
}

void
Token::addUnified (const class Valuation& valuation)
{
  assert (myUnifier.varMult);
  assert (!myUnifiedUndefined);
  const class VariableSet& vars = *myUnifier.vars;
  for (VariableSet::const_iterator i = vars.begin (); i != vars.end (); i++) {
    if (valuation.getValue (**i)) {
      if (!myUnifiedUndefined) myUnifiedUndefined = new class VariableSet;
      myUnifiedUndefined->insert (**i);
    }
  }
}

bool
Token::isUnified (const class VariableDefinition& var) const
{
  return myUnifiedUndefined && myUnifiedUndefined->contains (var);
}

#ifndef NDEBUG
# include "VariableDefinition.h"

void
Token::assertUndefined (const class Valuation& valuation) const
{
  const class VariableSet& vars = *myUnifier.vars;
  for (VariableSet::const_iterator i = vars.begin (); i != vars.end (); i++)
    assert (!valuation.getValue (**i) || (*i)->isUndefined ());
}

void
Token::assertDefined (const class Valuation& valuation) const
{
  const class VariableSet& vars = *myUnifier.vars;
  for (VariableSet::const_iterator i = vars.begin (); i != vars.end (); i++)
    assert (!!valuation.getValue (**i));
}
#endif // NDEBUG
